Imports Microsoft.EntityFrameworkCore
Imports Microsoft.EntityFrameworkCore.ChangeTracking
Imports SamuraiApp.Domain

Namespace SamuraiApp.Data
  Public Class ConnectedData
	Private _context As SamuraiContext

	Public Sub New()
	  _context = New SamuraiContext
	  _context.Database.EnsureCreated()
	End Sub

	Public Function CreateNewSamurai() As Samurai
	  'battles (many to many) will not be involved
	  Dim samurai = New Samurai With {.Name = "New Samurai"}
	  _context.Samurais.Add(samurai)
	  Return samurai
	End Function

	Public Function SamuraisListInMemory() As LocalView(Of Samurai)
	  If _context.Samurais.Local.Count = 0 Then
		_context.Samurais.ToList()
	  End If
	  Return _context.Samurais.Local
	End Function

	Public Function LoadSamuraiGraph(ByVal samuraiId As Integer) As Samurai
	  Dim samurai = _context.Samurais.Find(samuraiId) 'gets from tracker if it's there
	  _context.Entry(samurai).Reference(Function(s) s.SecretIdentity).Load()
	  _context.Entry(samurai).Collection(Function(s) s.Quotes).Load()
	  Return samurai
	End Function


	Public Sub SaveChanges(ByVal typeBeingEdited As Type)
	  _context.SaveChanges()
	  If typeBeingEdited Is GetType(Samurai) Then
		If _context.Samurais.Local.Any Then
		  SamuraisListInMemory.ToList.ForEach(Function(s) s.IsDirty = False)
		End If
	  End If
	  If typeBeingEdited Is GetType(Battle) Then
		If _context.Battles.Local.Any Then
		  BattlesListInMemory.ToList.ForEach(Function(s) s.IsDirty = False)
		End If
	  End If

	End Sub

	Public Function BattlesListInMemory() As LocalView(Of Battle)
	  If _context.Battles.Local.Count = 0 Then
		_context.Battles.ToList()
	  End If
	  Return _context.Battles.Local
	End Function

	Public Function SamuraisNotInBattle(ByVal battleId As Integer) As List(Of Samurai)
	  Dim existingSamurais = _context.SamuraiBattles.Where(Function(sb) sb.BattleId = battleId).Select(Function(sb) sb.SamuraiId).ToList
            Dim samurais = _context.Samurais.AsNoTracking.Where(Function(s) (Not existingSamurais.Contains(s.Id))).ToList
            Return samurais
	End Function

	Public Function LoadBattleGraph(ByVal battleId As Integer) As Battle
            Dim battle = _context.Battles.Find(battleId) 'gets from tracker if its there
            _context.Entry(battle).Collection(Function(b) b.SamuraiBattles).Query.Include(Function(sb) sb.Samurai).Load()
	  Return battle
	End Function

	Public Sub AddSamuraiBattle(ByVal samuraiBattle As SamuraiBattle)
	  'presumes samurai and battle always already exist
	 _context.Entry(samuraiBattle).State =EntityState.Added
	End Sub



	Public Sub RevertBattleChanges(ByVal id As Integer)
	  'this is the simplest way. 
	  'Maybe later versions of EF will make it easier
	 _context = New SamuraiContext
	End Sub

	Public Function CreateNewBattle() As Battle
	  'samurais (many to many) will not be involved
	  Dim battle = New Battle With {.Name = "New Battle"}
	  _context.Battles.Add(battle)
	  Return battle
	End Function
  End Class
End Namespace
