Imports System.Configuration
Imports Microsoft.EntityFrameworkCore
Imports SamuraiApp.Domain

Namespace SamuraiApp.Data
  Public Class SamuraiContext
	  Inherits DbContext
	Public Property Samurais As DbSet(Of Samurai)
	Public Property Battles As DbSet(Of Battle)
	Public Property Quotes As DbSet(Of Quote)
	Public Property SamuraiBattles As DbSet(Of SamuraiBattle)

	Protected Overrides Sub OnModelCreating(ByVal modelBuilder As ModelBuilder)

	  modelBuilder.Entity(Of SamuraiBattle)().HasKey(Function(s) New With {Key s.BattleId, Key s.SamuraiId})

	  For Each entityType In modelBuilder.Model.GetEntityTypes
		modelBuilder.Entity(entityType.Name).Property(Of Date)("LastModified")
		modelBuilder.Entity(entityType.Name).Ignore("IsDirty")
	  Next entityType
	End Sub

	Public Sub New(ByVal options As DbContextOptions)
		MyBase.New(options)
	End Sub

	Public Sub New()
	End Sub

	Protected Overrides Sub OnConfiguring(ByVal optionsBuilder As DbContextOptionsBuilder)
	  If Not optionsBuilder.IsConfigured Then
		Dim settings = ConfigurationManager.ConnectionStrings
		Dim connectionString = settings("productionDb").ConnectionString
		optionsBuilder.UseSqlServer(connectionString, Function(options) options.MaxBatchSize(30))
		optionsBuilder.EnableSensitiveDataLogging()
	  End If
	End Sub

        Public Overrides Function SaveChanges() As Integer
            For Each Entry As ChangeTracking.EntityEntry In ChangeTracker.Entries.Where(Function(e) e.State = EntityState.Added OrElse e.State = EntityState.Modified)
                Entry.Property("LastModified").CurrentValue = Date.Now
            Next Entry

            Return MyBase.SaveChanges
        End Function
    End Class
End Namespace
