Imports SamuraiApp.Data
Imports SamuraiApp.Domain

Namespace SamuraiWpf
  ''' <summary>
  '''   Interaction logic for Battles.xaml
  ''' </summary>
  Partial Public Class BattlesWindow
	  Inherits Window
	Private ReadOnly _repo As New ConnectedData
	Private _availableSamurais As List(Of Samurai)
	Private _battleViewSource As ObjectDataProvider
	Private _currentBattle As Battle
	Private _isListChanging As Boolean
	Private _isLoading As Boolean
	Private _startPoint As Point

	Public Sub New()
	  InitializeComponent()
	End Sub

	Private Sub Window_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  _isLoading = True
	  battleListBox.ItemsSource = _repo.BattlesListInMemory
	  _battleViewSource = CType(FindResource("battleViewSource"), ObjectDataProvider)
	  _isLoading = False
	  battleListBox.SelectedIndex = 0
	End Sub

	Private Sub battleListBox_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
	  If _currentBattle IsNot Nothing Then
		If _currentBattle.IsDirty Then
		  Select Case MessageBox.Show("Save current battle?", "Battle Builder", MessageBoxButton.YesNoCancel)
			Case MessageBoxResult.Cancel
			  Exit Sub

			Case MessageBoxResult.Yes
			  _repo.SaveChanges(_currentBattle.GetType)

			Case MessageBoxResult.No
			  _currentBattle.IsDirty = False
			  _repo.RevertBattleChanges(_currentBattle.Id)

		  End Select
		End If
	  End If

	  If Not _isLoading Then
		_isListChanging = True
		_currentBattle = _repo.LoadBattleGraph(CInt(Fix(battleListBox.SelectedValue)))
		_battleViewSource.ObjectInstance = _currentBattle
		_availableSamurais = _repo.SamuraisNotInBattle(_currentBattle.Id)
		samuraisNotInBattle.ItemsSource = _availableSamurais

		_isListChanging = False
	  End If
	End Sub



	Private Sub button_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  _repo.SaveChanges(_currentBattle.GetType)
	End Sub

	#Region "AddSamuraiToBattle"

	Private Sub samuraisNotInBattle_MouseDown(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
	  _startPoint = e.GetPosition(Nothing)
	End Sub

	Private Sub samuraisNotInBattle_PreviewMouseMove(ByVal sender As Object, ByVal e As MouseEventArgs)
	  GatherSelectedItemForMove(sender, e)
	End Sub

	Private Sub samuraisInBattle_DragEnter(ByVal sender As Object, ByVal e As DragEventArgs)
	  IgnoreNonSamuraiItem(sender, e)
	End Sub

	Private Sub AddDroppedSamuraiToBattle(ByVal sender As Object, ByVal e As DragEventArgs)
	  If e.Data.GetDataPresent(GetType(Samurai)) Then
		Dim samurai = TryCast(e.Data.GetData(GetType(Samurai)), Samurai)
		Dim samuraiBattle = New SamuraiBattle With {.Battle = _currentBattle, .Samurai = samurai}
		_repo.AddSamuraiBattle(samuraiBattle)
		_availableSamurais.Remove(samurai)
		NoteSamuraiMove()
	  End If
	End Sub

	#End Region ' AddSamuraiToBattle

	#Region "RemoveSamuraiFromBattle"

	Private Sub samuraisInBattle_MouseDown(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
	  _startPoint = e.GetPosition(Nothing)
	End Sub

	Private Sub samuraisInBattle_PreviewMouseMove(ByVal sender As Object, ByVal e As MouseEventArgs)
	  GatherSelectedItemForMove(sender, e)
	End Sub

	Private Sub samuraisNotInBattle_DragEnter(ByVal sender As Object, ByVal e As DragEventArgs)
	  IgnoreNonSamuraiItem(sender, e)
	End Sub

	Private Sub RemoveDroppedSamuraiFromBattle(ByVal sender As Object, ByVal e As DragEventArgs)
	  If e.Data.GetDataPresent(GetType(SamuraiBattle)) Then
		Dim samuraiBattle = TryCast(e.Data.GetData(GetType(SamuraiBattle)), SamuraiBattle)
		_currentBattle.SamuraiBattles.Remove(samuraiBattle)

		_availableSamurais.Add(samuraiBattle.Samurai)
		NoteSamuraiMove()
	  End If
	End Sub

	#End Region ' RemoveSamuraiFromBattle

	Private Sub nameTextBox_TextChanged(ByVal sender As Object, ByVal e As TextChangedEventArgs)
	  If (Not _isLoading) AndAlso (Not _isListChanging) Then
		_currentBattle.IsDirty = True
	  End If
	End Sub

	Private Sub startDateDatePicker_SelectedDateChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
	  If (Not _isLoading) AndAlso (Not _isListChanging) Then
		_currentBattle.IsDirty = True
	  End If
	End Sub

	Private Sub endDateDatePicker_SelectedDateChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
	  If (Not _isLoading) AndAlso (Not _isListChanging) Then
		_currentBattle.IsDirty = True
	  End If

	End Sub

	Private Sub NoteSamuraiMove()
	  samuraisNotInBattle.Items.Refresh()
	  samuraisInBattle.Items.Refresh()
	  _currentBattle.IsDirty = True
	  battleListBox.Items.Refresh()
	End Sub

	Private Sub GatherSelectedItemForMove(ByVal sender As Object, ByVal e As MouseEventArgs)
	  Dim mousePos = e.GetPosition(Nothing)
	  Dim diff = _startPoint - mousePos
	  If e.LeftButton = MouseButtonState.Pressed AndAlso (Math.Abs(diff.X) > SystemParameters.MinimumHorizontalDragDistance OrElse Math.Abs(diff.Y) > SystemParameters.MinimumVerticalDragDistance) Then
		Dim listBox = TryCast(sender, ListBox)
		Dim listBoxItem = listBox.SelectedItem
		If listBoxItem IsNot Nothing Then
		  DragDrop.DoDragDrop(listBox, listBoxItem, DragDropEffects.Move)
		End If
	  End If
	End Sub

	Private Shared Sub IgnoreNonSamuraiItem(ByVal sender As Object, ByVal e As DragEventArgs)
	  If (Not e.Data.GetDataPresent(GetType(Samurai))) OrElse sender Is e.Source Then
		e.Effects = DragDropEffects.None
	  End If
	End Sub

	Private Sub New_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  _isListChanging = True
	  _currentBattle = _repo.CreateNewBattle
	  _battleViewSource.ObjectInstance = _currentBattle
	  battleListBox.SelectedItem = _currentBattle
	  samuraisInBattle.ItemsSource = _currentBattle.SamuraiBattles
	  _isListChanging = False
	  _currentBattle.IsDirty = True
	End Sub

	Private Sub GoToSamurais_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)

	  Dim samuraisWindow = New MainWindow 'create your new form.
	  samuraisWindow.Show() 'show the new form.
	  Close() 'only if you want to close the current form.
	End Sub
  End Class
End Namespace
