Imports System.Collections.ObjectModel
Imports SamuraiApp.Data
Imports SamuraiApp.Domain

Namespace SamuraiWpf
  Partial Public Class MainWindow
	  Inherits Window
	Private ReadOnly _repo As New ConnectedData
	Private _currentSamurai As Samurai
	Private _isListChanging As Boolean
	Private _isLoading As Boolean
	Private _samuraiViewSource As ObjectDataProvider

	Public Sub New()
	  InitializeComponent()
	End Sub

	Private Sub Window_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  _isLoading = True
	  samuraiListBox.ItemsSource = _repo.SamuraisListInMemory
	  _samuraiViewSource = CType(FindResource("SamuraiViewSource"), ObjectDataProvider)
	  _isLoading = False
	  samuraiListBox.SelectedIndex = 0
	End Sub

	Private Sub SamuraiListBox_OnSelectionChangedListBox_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
	  If Not _isLoading Then
		_isListChanging = True
		_currentSamurai = _repo.LoadSamuraiGraph(CInt(Fix(samuraiListBox.SelectedValue)))
		_samuraiViewSource.ObjectInstance = _currentSamurai
		_isListChanging = False
	  End If
	End Sub

	Private Sub Save_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  _repo.SaveChanges(_currentSamurai.GetType)
	  'samuraiListBox.ItemsSource = _repo.SamuraisListInMemory;

	End Sub

	Private Sub realNameTextBox_TextChanged(ByVal sender As Object, ByVal e As TextChangedEventArgs)
	  If (Not _isLoading) AndAlso (Not _isListChanging) Then
		If _currentSamurai.SecretIdentity Is Nothing Then
		  _currentSamurai.SecretIdentity = New SecretIdentity
		End If
		_currentSamurai.SecretIdentity.RealName = (CType(sender, TextBox)).Text
		_currentSamurai.IsDirty = True
	  End If
	End Sub

	Private Sub New_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)

	  _currentSamurai = _repo.CreateNewSamurai
	  _samuraiViewSource.ObjectInstance = _currentSamurai
	  samuraiListBox.SelectedItem = _currentSamurai
	End Sub

	Private Sub nameTextBox_TextChanged(ByVal sender As Object, ByVal e As TextChangedEventArgs)
	  If (Not _isLoading) AndAlso (Not _isListChanging) Then
		_currentSamurai.IsDirty = True
	  End If
	End Sub

	Private Sub nameTextBox_LostFocus(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  samuraiListBox.Items.Refresh()
	End Sub

	Private Sub quotesDataGrid_CellEditEnding(ByVal sender As Object, ByVal e As DataGridCellEditEndingEventArgs)
	  If (Not _isLoading) AndAlso (Not _isListChanging) Then
		_currentSamurai.IsDirty = True
	  End If
	End Sub

	Private Sub gotoBattles_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  Dim battlesWindow = New BattlesWindow 'create your new form.
	  battlesWindow.Show() 'show the new form.
	  Me.Close() 'only if you want to close the current form.
	End Sub
  End Class
End Namespace
