Imports System.Linq
Imports Microsoft.EntityFrameworkCore
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports SamuraiApp.Data
Imports SamuraiApp.Domain

Namespace UnitTestProject
  <TestClass>
  Public Class SamuraiTests

	<TestMethod>
	Public Sub CanInsertSamuraiIntoSqlServer()
	  Dim optionsBuilder = New DbContextOptionsBuilder
	  optionsBuilder.UseSqlServer ("Server = (localdb)\mssqllocaldb; Database = TestDb; Trusted_Connection = True; ")
	  Using context = New SamuraiContext(optionsBuilder.Options)
		context.Database.EnsureDeleted()
		context.Database.EnsureCreated()
		Dim samurai = New Samurai
		Debug.WriteLine($"Default Samurai Id {samurai.Id}")
		context.Samurais.Add(samurai)
		Dim efDefaultId = samurai.Id
		Debug.WriteLine($"EF assigned Samurai Id {samurai.Id}")
		context.SaveChanges()
		Debug.WriteLine($"DB assigned Samurai Id {samurai.Id}")
		Assert.AreNotEqual(efDefaultId, samurai.Id)
	  End Using
	End Sub
	<TestMethod>
	Public Sub CanInsertSamuraiIntoSqlite()
	  Dim optionsBuilder = New DbContextOptionsBuilder
	  optionsBuilder.UseSqlite ("Filename=test.db ")
	  Using context = New SamuraiContext(optionsBuilder.Options)
		context.Database.EnsureDeleted()
		context.Database.EnsureCreated()
		Dim samurai = New Samurai
		Debug.WriteLine($"Default Samurai Id {samurai.Id}")
		context.Samurais.Add(samurai)
		Dim efDefaultId = samurai.Id
		Debug.WriteLine($"EF assigned Samurai Id {samurai.Id}")

		context.SaveChanges()
		Debug.WriteLine($"DB assigned Samurai Id {samurai.Id}")
		Assert.AreNotEqual(efDefaultId, samurai.Id)
	  End Using
	End Sub

	<TestMethod>
	Public Sub CanInsertSamuraiWithSaveChanges()
	  Dim optionsBuilder = New DbContextOptionsBuilder
	  optionsBuilder.UseInMemoryDatabase()
	  Using context = New SamuraiContext(optionsBuilder.Options)
		Dim samurai = New Samurai
		samurai.Name = "Julie"
		samurai.Id = 36
		Debug.WriteLine($"Default Samurai Id {samurai.Id}")
		context.Samurais.Add(samurai)
		Debug.WriteLine($"EF assigned Samurai Id {samurai.Id}")
		context.SaveChanges()
		Debug.WriteLine($"DB assigned Samurai Id {samurai.Id}")
	  End Using
	  Using context2 = New SamuraiContext(optionsBuilder.Options)
		Assert.AreEqual(1, context2.Samurais.Count)
	  End Using
	End Sub
  End Class
End Namespace
